/*
    FreeRTOS V8.2.3 - Copyright (C) 2015 Real Time Engineers Ltd.

 * Authors: Benjamin Lin and John Hyunh
    All rights reserved

    VISIT http://www.FreeRTOS.org TO ENSURE YOU ARE USING THE LATEST VERSION.

    This file is part of the FreeRTOS distribution.

    FreeRTOS is free software; you can redistribute it and/or modify it under
    the terms of the GNU General Public License (version 2) as published by the
    Free Software Foundation >>>> AND MODIFIED BY <<<< the FreeRTOS exception.

    ***************************************************************************
    >>!   NOTE: The modification to the GPL is included to allow you to     !<<
    >>!   distribute a combined work that includes FreeRTOS without being   !<<
    >>!   obliged to provide the source code for proprietary components     !<<
    >>!   outside of the FreeRTOS kernel.                                   !<<
    ***************************************************************************

    FreeRTOS is distributed in the hope that it will be useful, but WITHOUT ANY
    WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
    FOR A PARTICULAR PURPOSE.  Full license text is available on the following
    link: http://www.freertos.org/a00114.html

    ***************************************************************************
     *                                                                       *
     *    FreeRTOS provides completely free yet professionally developed,    *
     *    robust, strictly quality controlled, supported, and cross          *
     *    platform software that is more than just the market leader, it     *
     *    is the industry's de facto standard.                               *
     *                                                                       *
     *    Help yourself get started quickly while simultaneously helping     *
     *    to support the FreeRTOS project by purchasing a FreeRTOS           *
     *    tutorial book, reference manual, or both:                          *
     *    http://www.FreeRTOS.org/Documentation                              *
     *                                                                       *
    ***************************************************************************

    http://www.FreeRTOS.org/FAQHelp.html - Having a problem?  Start by reading
    the FAQ page "My application does not run, what could be wrong?".  Have you
    defined configASSERT()?

    http://www.FreeRTOS.org/support - In return for receiving this top quality
    embedded software for free we request you assist our global community by
    participating in the support forum.

    http://www.FreeRTOS.org/training - Investing in training allows your team to
    be as productive as possible as early as possible.  Now you can receive
    FreeRTOS training directly from Richard Barry, CEO of Real Time Engineers
    Ltd, and the world's leading authority on the world's leading RTOS.

    http://www.FreeRTOS.org/plus - A selection of FreeRTOS ecosystem products,
    including FreeRTOS+Trace - an indispensable productivity tool, a DOS
    compatible FAT file system, and our tiny thread aware UDP/IP stack.

    http://www.FreeRTOS.org/labs - Where new FreeRTOS products go to incubate.
    Come and try FreeRTOS+TCP, our new open source TCP/IP stack for FreeRTOS.

    http://www.OpenRTOS.com - Real Time Engineers ltd. license FreeRTOS to High
    Integrity Systems ltd. to sell under the OpenRTOS brand.  Low cost OpenRTOS
    licenses offer ticketed support, indemnification and commercial middleware.

    http://www.SafeRTOS.com - High Integrity Systems also provide a safety
    engineered and independently SIL3 certified version for use in safety and
    mission critical applications that require provable dependability.

    1 tab == 4 spaces!
*/

/******************************************************************************
 * NOTE 1:  This project provides three demo applications.  A simple blinky
 * style project, a more comprehensive test and demo application, and an
 * lwIP example.  The mainSELECTED_APPLICATION setting in main.c is used to
 * select between the three.  See the notes on using mainSELECTED_APPLICATION
 * in main.c.  This file implements the simply blinky style version.
 *
 * NOTE 2:  This file only contains the source code that is specific to the
 * basic demo.  Generic functions, such FreeRTOS hook functions, and functions
 * required to configure the hardware are defined in main.c.
 ******************************************************************************
 *
 * main_blinky() creates one queue, and two tasks.  It then starts the
 * scheduler.
 *
 * The Queue Send Task:
 * The queue send task is implemented by the prvQueueSendTask() function in
 * this file.  prvQueueSendTask() sits in a loop that causes it to repeatedly
 * block for 200 milliseconds, before sending the value 100 to the queue that
 * was created within main_blinky().  Once the value is sent, the task loops
 * back around to block for another 200 milliseconds...and so on.
 *
 * The Queue Receive Task:
 * The queue receive task is implemented by the prvQueueReceiveTask() function
 * in this file.  prvQueueReceiveTask() sits in a loop where it repeatedly
 * blocks on attempts to read data from the queue that was created within
 * main_blinky().  When data is received, the task checks the value of the
 * data, and if the value equals the expected 100, toggles an LED.  The 'block
 * time' parameter passed to the queue receive function specifies that the
 * task should be held in the Blocked state indefinitely to wait for data to
 * be available on the queue.  The queue receive task will only leave the
 * Blocked state when the queue send task writes to the queue.  As the queue
 * send task writes to the queue every 200 milliseconds, the queue receive
 * task leaves the Blocked state every 200 milliseconds, and therefore toggles
 * the LED every 200 milliseconds.
 */



/* Xilinx includes. */
#include "xgpio.h"

/* Kernel includes. */
#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"
#include "string.h"


#define partstNUM_LEDS			( 1 )
#define partstDIRECTION_OUTPUT	( 1 )
#define partstOUTPUT_ENABLED	( 1 )
#define partstLED_OUTPUT		( 1 )
#define partstSW_INPUT		    ( 2 )

/* Standard demo includes. */
#include "partest.h"

/* Priorities at which the tasks are created. */
#define mainQUEUE_RECEIVE_TASK_PRIORITY		( tskIDLE_PRIORITY + 2 )
#define	mainQUEUE_SEND_TASK_PRIORITY		( tskIDLE_PRIORITY + 1 )

/* The rate at which data is sent to the queue.  The 200ms value is converted
to ticks using the portTICK_PERIOD_MS constant. */
#define mainQUEUE_SEND_FREQUENCY_MS			( 1 / portTICK_PERIOD_MS )

/* The number of items the queue can hold.  This is 1 as the receive task
will remove items as they are added, meaning the send task should always find
the queue empty. */
#define mainQUEUE_LENGTH					( 1 )

/* The LED toggled by the Rx task. */
#define mainTASK_LED						( 0 )


#include "xparameters.h"	/* SDK generated parameters */
#include "xsdps.h"		/* SD device driver */
#include "xil_printf.h"
#include "ff.h"
#include "xil_cache.h"

/************************** Constant Definitions *****************************/


/**************************** Type Definitions *******************************/

/***************** Macros (Inline Functions) Definitions *********************/

/************************** Function Prototypes ******************************/
int FfsSdPolledExample(void);

/************************** Variable Definitions *****************************/
static FIL fil_r;		/* File object */
static FIL fil_g;
static FIL fil_b;
static FATFS fatfs;
//static char FileName[32] = "FRED2.BIN";
static char *SD_File_R;
static char *SD_File_G;
static char *SD_File_B;

static XGpio xGpio;
static XGpio rgb;


#ifdef __ICCARM__
#pragma data_alignment = 32

u8 SourceAddress[10*1024*1024];
#pragma data_alignment = 4
#else
u8 Dest1[10*1024*1024];
u8 Dest2[10*1024*1024];
u8 Dest3[10*1024*1024];

u8 DestinationAddress[10*1024*1024] __attribute__ ((aligned(32)));
u8 SourceAddress[10*1024*1024] __attribute__ ((aligned(32)));
#endif

#define TEST 7
/*-----------------------------------------------------------*/

/*
 * The tasks as described in the comments at the top of this file.
 */
static void prvQueueReceiveTask( void *pvParameters );
static void prvQueueSendTask( void *pvParameters );

/*-----------------------------------------------------------*/

/* The queue used by both tasks. */
static QueueHandle_t xQueue = NULL;

/*-----------------------------------------------------------*/
typedef struct RGB{

	int redArray[6400];
	int blueArray[6400];
	int greenArray[6400];
}RGB;


void loadMudkip(RGB *mudkip, char* fileR, char* fileG, char* fileB){
	FRESULT Res;
	UINT NumBytesRead;

	Res = f_open(&fil_r, fileR,  FA_READ);
	Res = f_open(&fil_g, fileG,  FA_READ);
	Res = f_open(&fil_b, fileB,  FA_READ);


	if (Res) {
		return XST_FAILURE;
	}


	/*
	 * Pointer to beginning of file .
	 */
	Res = f_lseek(&fil_r, 0);
	Res = f_lseek(&fil_g, 0);
	Res = f_lseek(&fil_b, 0);

	if (Res) {
		return XST_FAILURE;
	}

	/*
	 * Read data from file.
	 */
	Res = f_read(&fil_r, (void*)Dest1, fil_r.fsize,
			&NumBytesRead);

	Res = f_read(&fil_g, (void*)Dest2, fil_g.fsize,
				&NumBytesRead);

	Res = f_read(&fil_b, (void*)Dest3, fil_b.fsize,
				&NumBytesRead);
	if (Res) {
		return XST_FAILURE;
	}


    char *p2 = Dest1;
    int lines = 0;
	while( lines < 79){
		if (*p2 == '\n'){
			lines++;
			*p2 = ',';
		}
		p2++;
	}

	 char *p22 = Dest2;
	    lines = 0;
		while( lines < 79){
			if (*p22 == '\n'){
				lines++;
				*p22 = ',';
			}
			p22++;
		}

		 char *p222 = Dest3;
		    lines = 0;
			while( lines < 79){
				if (*p222 == '\n'){
					lines++;
					*p222 = ',';
				}
				p222++;
			}





	int count = 0;
	char *pt;

	pt = strtok(Dest1, ",");

	while(pt != NULL && count < 6400){
		int a = atoi(pt);
		pt = strtok(NULL,",");
		mudkip->redArray[count] = a;
		count++;
	}

	count = 0;
	pt = strtok(Dest2, ",");

		while(pt != NULL && count < 6400){
			int a = atoi(pt);
			pt = strtok(NULL,",");
			mudkip->greenArray[count] = a;
			count++;
		}

		count = 0;
			pt = strtok(Dest3, ",");

				while(pt != NULL && count < 6400){
					int a = atoi(pt);
					pt = strtok(NULL,",");
					mudkip->blueArray[count] = a;
					count++;
				}

				/*
				 * Close file.
				 */
				Res = f_close(&fil_r);
				Res = f_close(&fil_g);
				Res = f_close(&fil_b);
				if (Res) {
					return XST_FAILURE;
				}
				return;



}

int FfsSdPolledExample(void)
{

	BaseType_t xStatus;
		//XGpio_Config *pxConfigPtr;
		/* Initialise the GPIO driver. */
		//pxConfigPtr = XGpioPs_LookupConfig( XPAR_XGPIOPS_0_DEVICE_ID );

		xStatus = XGpio_Initialize( &xGpio, XPAR_AXI_GPIO_0_DEVICE_ID );
		xStatus = XGpio_Initialize( &rgb, XPAR_AXI_GPIO_1_DEVICE_ID );
		configASSERT( xStatus == XST_SUCCESS );
		( void ) xStatus; /* Remove compiler warning if configASSERT() is not defined. */

		/* Enable outputs and set low. */
		XGpio_SetDataDirection( &rgb, 1, partstDIRECTION_OUTPUT );
		XGpio_SetDataDirection( &rgb, 2, partstDIRECTION_OUTPUT );
		XGpio_SetDataDirection( &xGpio, 1, partstDIRECTION_OUTPUT );
		XGpio_SetDataDirection( &xGpio, 2, partstDIRECTION_OUTPUT );


	FRESULT Res;
		UINT NumBytesWritten;
		u32 BuffCnt;
		u32 FileSize = (8*1024*1024);
	UINT NumBytesRead;
	TCHAR *Path = "0:/";

	/*
	 * Register volume work area, initialize device
	 */
	Res = f_mount(0, &fatfs);

		if (Res != FR_OK) {
			return XST_FAILURE;
		}

	/*
	 * Open file with required permissions.
	 * Here - Creating new file with read/write permissions. .
	 * To open file with write permissions, file system should not
	 * be in Read Only mode.
	 */


	RGB mudkips[20] = {0};

	int k = 0;

	loadMudkip(&mudkips[0],"MUD0R.BIN","MUD0G.BIN","MUD0B.BIN");
	loadMudkip(&mudkips[1],"MUD1R.BIN","MUD1G.BIN","MUD1B.BIN");
	loadMudkip(&mudkips[2],"MUD2R.BIN","MUD2G.BIN","MUD2B.BIN");
	loadMudkip(&mudkips[3],"MUD3R.BIN","MUD3G.BIN","MUD3B.BIN");
	loadMudkip(&mudkips[4],"MUD4R.BIN","MUD4G.BIN","MUD4B.BIN");
	loadMudkip(&mudkips[5],"MUD5R.BIN","MUD5G.BIN","MUD5B.BIN");
	loadMudkip(&mudkips[6],"MUD6R.BIN","MUD6G.BIN","MUD6B.BIN");
	loadMudkip(&mudkips[7],"MUD7R.BIN","MUD7G.BIN","MUD7B.BIN");
	loadMudkip(&mudkips[8],"MUD8R.BIN","MUD8G.BIN","MUD8B.BIN");
	loadMudkip(&mudkips[9],"MUD9R.BIN","MUD9G.BIN","MUD9B.BIN");

	loadMudkip(&mudkips[10],"SUN0R.BIN","SUN0G.BIN","SUN0B.BIN");
		loadMudkip(&mudkips[11],"SUN1R.BIN","SUN1G.BIN","SUN1B.BIN");
		loadMudkip(&mudkips[12],"SUN2R.BIN","SUN2G.BIN","SUN2B.BIN");
		loadMudkip(&mudkips[13],"SUN3R.BIN","SUN3G.BIN","SUN3B.BIN");
		loadMudkip(&mudkips[14],"SUN4R.BIN","SUN4G.BIN","SUN4B.BIN");
		loadMudkip(&mudkips[15],"SUN5R.BIN","SUN5G.BIN","SUN5B.BIN");
		loadMudkip(&mudkips[16],"SUN6R.BIN","SUN6G.BIN","SUN6B.BIN");
		loadMudkip(&mudkips[17],"SUN7R.BIN","SUN7G.BIN","SUN7B.BIN");
		loadMudkip(&mudkips[18],"SUN8R.BIN","SUN8G.BIN","SUN8B.BIN");
		loadMudkip(&mudkips[19],"SUN9R.BIN","SUN9G.BIN","SUN9B.BIN");



while(1){

    int mudKipGo = 0;

    while(mudKipGo < 20){

	int z = 64;
	int p = 40008;
	int m = 00;
	int index = 0;
	int row = 0;

	while(p < 74752){
	while( z < 144){

			index =  m + row;
			XGpio_DiscreteWrite(&xGpio, 1, z+p);
			XGpio_DiscreteWrite(&xGpio, 2, mudkips[mudKipGo].redArray[index]/8);
			XGpio_DiscreteWrite(&rgb, 1, mudkips[mudKipGo].greenArray[index]/4);
			XGpio_DiscreteWrite(&rgb, 2, mudkips[mudKipGo].blueArray[index]/8);
			index++;
			if ( index == 6400){
			   z = 2342424;
			   p = 242424;
			}
			m += 80;
			if (m == 6400){
				m = 0;
				row++;
			}

		z++;
	}
    z = 64;
	p += 256;
}

	mudKipGo++;

    int gogogo = 0;

    while(gogogo++ < 3){
	int qwert = 0;

	while(qwert < 999999){
		qwert++;
	}
	qwert = 0;
	while(qwert < 999999){
			qwert++;
		}
	qwert = 0;
	while(qwert < 999999){
			qwert++;
		}
	qwert = 0;
	while(qwert < 999999){
			qwert++;
		}
	qwert = 0;
	while(qwert < 999999){
			qwert++;
		}

    }
    }
}




 return 0;

}


void main_blinky( void )
{



	FfsSdPolledExample();





	/* Create the queue. */
	xQueue = xQueueCreate( mainQUEUE_LENGTH, sizeof( uint32_t ) );

	if( xQueue != NULL )
	{
		/* Start the two tasks as described in the comments at the top of this
		file. */
		xTaskCreate( prvQueueReceiveTask,				/* The function that implements the task. */
					"Rx", 								/* The text name assigned to the task - for debug only as it is not used by the kernel. */
					configMINIMAL_STACK_SIZE, 			/* The size of the stack to allocate to the task. */
					NULL, 								/* The parameter passed to the task - not used in this case. */
					mainQUEUE_RECEIVE_TASK_PRIORITY, 	/* The priority assigned to the task. */
					NULL );								/* The task handle is not required, so NULL is passed. */

	//	xTaskCreate( prvQueueSendTask, "TX", configMINIMAL_STACK_SIZE, NULL, mainQUEUE_SEND_TASK_PRIORITY, NULL );

		/* Start the tasks and timer running. */
		vTaskStartScheduler();

	}

	/* If all is well, the scheduler will now be running, and the following
	line will never be reached.  If the following line does execute, then
	there was either insufficient FreeRTOS heap memory available for the idle
	and/or timer tasks to be created, or vTaskStartScheduler() was called from
	User mode.  See the memory management section on the FreeRTOS web site for
	more details on the FreeRTOS heap http://www.freertos.org/a00111.html.  The
	mode from which main() is called is set in the C start up code and must be
	a privileged mode (not user mode). */
	for( ;; );
}
/*-----------------------------------------------------------*/

static void prvQueueSendTask( void *pvParameters )
{
TickType_t xNextWakeTime;
const unsigned long ulValueToSend = 100UL;

	/* Remove compiler warning about unused parameter. */
	( void ) pvParameters;

	/* Initialise xNextWakeTime - this only needs to be done once. */
	xNextWakeTime = xTaskGetTickCount();

	for( ;; )
	{
		/* Place this task in the blocked state until it is time to run again. */
		vTaskDelayUntil( &xNextWakeTime, mainQUEUE_SEND_FREQUENCY_MS );

		/* Send to the queue - causing the queue receive task to unblock and
		toggle the LED.  0 is used as the block time so the sending operation
		will not block - it shouldn't need to block as the queue should always
		be empty at this point in the code. */
		xQueueSend( xQueue, &ulValueToSend, 0U );
	}
}
/*-----------------------------------------------------------*/

static void prvQueueReceiveTask( void *pvParameters )
{
unsigned long ulReceivedValue;
const unsigned long ulExpectedValue = 100UL;

	/* Remove compiler warning about unused parameter. */
	( void ) pvParameters;

	for( ;; )
	{
		/* Wait until something arrives in the queue - this task will block
		indefinitely provided INCLUDE_vTaskSuspend is set to 1 in
		FreeRTOSConfig.h. */

		TickType_t xNextWakeTime;
		xNextWakeTime = xTaskGetTickCount();



		xQueueReceive( xQueue, &ulReceivedValue, portMAX_DELAY );

		/*  To get here something must have been received from the queue, but
		is it the expected value?  If it is, toggle the LED. */
			if( ulReceivedValue == ulExpectedValue )
			{


			//vParTestToggleLED( arr[i] );
			//vTaskDelayUntil( &xNextWakeTime, mainQUEUE_SEND_FREQUENCY_MS*2 );
			for (;;){
							vParTestToggleLED( mainTASK_LED );
							//vTaskDelayUntil( &xNextWakeTime, mainQUEUE_SEND_FREQUENCY_MS );
							vParTestToggleLED( mainTASK_LED );
							//vTaskDelayUntil( &xNextWakeTime, mainQUEUE_SEND_FREQUENCY_MS );

			}
			}
			ulReceivedValue = 0U;

	}
}
/*-----------------------------------------------------------*/

