----------------------------------------------------------------------------------
-- Company: Cal Poly San Luis Obispo
-- Engineer: Kevin Shibley and Konrad Antoniuk
-- 
-- Create Date:    01/25/2012 
-- Module Name:    ALU - Behavioral 
-- Project Name: 	 Lab05cpe233
-- Description: 	This module is an Arithmetic Logic Unit that is to be used in a
--		RAT CPU. Functionality includes addition and subtraction (with or without a
--		carry in), shifts and rotations (arithmetic and logical), and the ability to
--		compare or test two numbers.
--
-- Additional Comments: This module assumes that the input A to be the destination
--		register and the input B to be the source register.
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


entity ALU is
    Port ( A : in  STD_LOGIC_VECTOR (7 downto 0);
           B : in  STD_LOGIC_VECTOR (7 downto 0);
           C_IN : in  STD_LOGIC;
           SEL : in  STD_LOGIC_VECTOR (3 downto 0);
           SUM : out  STD_LOGIC_VECTOR (7 downto 0);
           C_FLAG : out  STD_LOGIC;
           Z_FLAG : out  STD_LOGIC);
end ALU;


architecture Behavioral of ALU is

signal out_temp : STD_LOGIC_VECTOR(8 downto 0);

begin


	selector: process(A,B,C_in,SEL)
		begin
			case SEL is
				when "0000" => out_temp <= '0' & A + B;  --ADD
				when "0001" => out_temp <= '0' & A + B + C_in;  --ADDC
				when "0010" => out_temp <= '0' & A - B;  --CMP
				when "0011" => out_temp <= '0' & A - B;  --SUB
				when "0100" => out_temp <= '0' & A - B - C_in; --SUBC
				when "0101" => out_temp <= '0' & (A and B);  --AND
				when "0110" => out_temp <= A(0) & A(7) & A(7) & A(6 downto 1);  --ASR
				when "0111" => out_temp <= '0' & (A xor B);  --EXOR
				when "1000" => out_temp <= A(7 downto 0) & C_IN;  --LSL
				when "1001" => out_temp <= A(0) & C_IN & A(7 downto 1);  --LSR
				when "1010" => out_temp <= '0' & (A or B);  --OR
				when "1011" => out_temp <= A(7 downto 0) & A(7);  --ROL
				when "1100" => out_temp <= A(0) & A(0) & A(7 downto 1);  --ROR
				when "1101" => out_temp <= '0' & (A and B);  --TEST
				when "1110" => out_temp <= '0' & B;  --MOV
				when others => out_temp <= '0' & B; --MOV instruction
			end case;
	end process selector;
	
	SUM <= out_temp(7 downto 0); 	-- extract the sum
	C_FLAG <= out_temp(8);		-- extract the carry out from the MSB of the signal
	
	check_zero: process(out_temp)		-- an unconditional check of zero flag
	begin										-- note: the flag register is only affected if the control unit allows for it
		if (out_temp(7 downto 0) = "00000000") then
			Z_FLAG <= '1';
		else
			Z_FLAG <= '0';
		end if;
	end process check_zero;	

end Behavioral;