----------------------------------------------------------------------------------
-- Company: CalPoly San Luis Obispo
-- Engineer: Kevin Shibley and Konrad Antoniuk
--
-- Module Name:    Debouncer - Behavioral 
--
-- Description: 
--
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: To make this debouncer into a pulse generator, simply assign
-- 	BTN_OUT <= '1' only during the transition from BTN_LOW to BTN_HIGH. Then assign
--		BTN_OUT the value 0 once inside BTN_HIGH.
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;


entity Debouncer is
    Port ( CLK : in  STD_LOGIC;
			  RST : in STD_LOGIC;
           BTN_IN : in  STD_LOGIC;
           BTN_OUT : out  STD_LOGIC);
end Debouncer;

architecture Behavioral of Debouncer is


   type state_type is (BTN_LOW, BTN_HIGH);
      signal PS,NS : state_type;
	
	
constant CLK_FREQ		: integer :=50000000;	
constant COUNT_MAX : integer := (CLK_FREQ/1000);
	
signal COUNT_LOW, COUNT_HIGH : integer range 0 to COUNT_MAX;
	
begin

--BTN_OUT <= BTN_IN;

   sync: process (CLK, NS, RST)
	begin
	   if (RST = '1') then
		   PS <= BTN_LOW;
		elsif (rising_edge(CLK)) then 
	      PS <= NS;
		end if;
	end process sync;


   Dbounce: process (PS, NS, CLK)
   begin
	if rising_edge(CLK) then
   case PS is
	
		   when BTN_LOW => 
		      if BTN_IN = '0' then
					COUNT_LOW <= 0;
					BTN_OUT <= '0';
				elsif BTN_IN = '1' and not (COUNT_LOW = COUNT_MAX) then
					COUNT_LOW <= COUNT_LOW + 1;
					BTN_OUT <= '0';
				elsif COUNT_LOW = COUNT_MAX then
					BTN_OUT <= '1';
					NS <= BTN_HIGH;
				end if;
			
			
			when BTN_HIGH =>
			BTN_OUT <= '1';
				if BTN_IN = '1' then
					COUNT_HIGH <= 0;
				elsif BTN_IN = '0' and not (COUNT_HIGH = COUNT_MAX) then
					COUNT_HIGH <= COUNT_HIGH + 1;
				elsif COUNT_HIGH = COUNT_MAX then
					NS <= BTN_LOW;
				end if;
		end case;
		end if;
	end process Dbounce;
	
end Behavioral;

