----------------------------------------------------------------------------------
-- Company: Cal Poly San Luis Obispo
-- Engineers: Kevin Shibley & Konrad Antoniuk
--
-- Module Name:    ScratchPad - Behavioral
-- Project Name:        Lab4part2
--
-- Description: This module implements a Random Access Memory (RAM) that will be
--      used in a RAT CPU. The memory is able to be written to only when the write
--      enable (WE) signal is received from the control unit in the CPU. The output
--      bus is set to high Z unless the output enable (OE) signal is received from
--      the control unit. Writes can only be performed upon the rising edge of the
--      clock, while reads can occur as long as OE is high.
--
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity ScratchPad is
   Port ( SP_ADRS   : in    STD_LOGIC_VECTOR (7 downto 0);
          SP_WE    : in     STD_LOGIC;
          SP_OE    : in     STD_LOGIC;
          SP_DATA: inout    STD_LOGIC_VECTOR (9 downto 0);
          CLK      : in     STD_LOGIC);
end ScratchPad;

architecture Behavioral of ScratchPad is
   TYPE memory is array (0 to 255) of std_logic_vector(9 downto 0);
   SIGNAL SP: memory := (others=>(others=>'0'));
begin

   process(clk, sp_we)
   begin
       if (rising_edge(clk) AND SP_WE = '1') then
           SP(conv_integer(SP_ADRS)) <= SP_DATA;
       end if;
   end process;

   SP_DATA <= SP(conv_integer(SP_ADRS)) when SP_OE='1' else "ZZZZZZZZZZ";

end Behavioral;