----------------------------------------------------------------------------------
-- Company: Cal Poly
-- Engineer: Kevin Shibley and Konrad Antoniuk
-- 
-- Create Date:    17:32:52 01/16/2012 
-- Design Name: 
-- Module Name:    lab02PCmodule - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: This program counter increments by default. It may also receive a
--			parallel load from the stack pointer or immediately from the prog_rom. The
--			counter will ony increment upon the rising edge of the clock if PC_LD is high.
-- 
--
-- Revision: A tri-state output has been added to this module. The output is always
-- assigned Pcount + 1 if OE is high, otherwise it is assigned high Z. The SEL has
-- also been given a fourth option to load the PC with 0x3FF; this is the value of the
-- interrupt vector in memory.
--
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
USE IEEE.NUMERIC_STD.ALL;

entity ProgramCounter is
    Port ( PC_INC : IN STD_LOGIC;
           PC_OE : in  STD_LOGIC;
           CLK : in  STD_LOGIC;
           RST : in STD_LOGIC;
			  PC_LD: in STD_LOGIC;
           D_IN : in  STD_LOGIC_VECTOR (9 downto 0);
           PC_TRI : out  STD_LOGIC_VECTOR (9 downto 0);
           pc_COUNT : out  STD_LOGIC_VECTOR (9 downto 0):= "0000000001");
end ProgramCounter;

architecture Behavioral of ProgramCounter is

signal pc_COUNT_temp : STD_LOGIC_VECTOR (9 downto 0) := "0000000001";


begin


ProcPCcount: process (CLK,RST,PC_LD,pc_COUNT_temp, PC_INC, PC_OE)
	begin
		if (RST = '1') then 
         pc_COUNT_temp <= "0000000001";
		elsif (rising_edge(CLK)) then
         if (PC_LD = '1') then
            pc_COUNT_temp <= D_IN;
         elsif (PC_INC = '1') then
            pc_COUNT_temp <= pc_COUNT_temp + '1';
         end if;
       end if;
       
      if (PC_OE = '1') then
         PC_TRI <= pc_COUNT_temp;
      else
         PC_TRI <= "ZZZZZZZZZZ";
      end if;
		
		pc_COUNT <= pc_COUNT_temp;
      
end process ProcPCcount;
	
end Behavioral;